from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .import_post_response_failure_items import ImportPostResponse_failure_items
    from .import_post_response_success_items import ImportPostResponse_success_items

@dataclass
class ImportPostResponse(Parsable):
    # The failure property
    failure: Optional[int] = None
    # The failure_items property
    failure_items: Optional[list[ImportPostResponse_failure_items]] = None
    # The success property
    success: Optional[int] = None
    # The success_items property
    success_items: Optional[list[ImportPostResponse_success_items]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ImportPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ImportPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ImportPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .import_post_response_failure_items import ImportPostResponse_failure_items
        from .import_post_response_success_items import ImportPostResponse_success_items

        from .import_post_response_failure_items import ImportPostResponse_failure_items
        from .import_post_response_success_items import ImportPostResponse_success_items

        fields: dict[str, Callable[[Any], None]] = {
            "failure": lambda n : setattr(self, 'failure', n.get_int_value()),
            "failure_items": lambda n : setattr(self, 'failure_items', n.get_collection_of_object_values(ImportPostResponse_failure_items)),
            "success": lambda n : setattr(self, 'success', n.get_int_value()),
            "success_items": lambda n : setattr(self, 'success_items', n.get_collection_of_object_values(ImportPostResponse_success_items)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_int_value("failure", self.failure)
        writer.write_collection_of_object_values("failure_items", self.failure_items)
        writer.write_int_value("success", self.success)
        writer.write_collection_of_object_values("success_items", self.success_items)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .assets_get_response import AssetsGetResponse

class AssetsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/assets/v2/projects/{projectId}/assets
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new AssetsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/assets/v2/projects/{projectId}/assets{?cursorState*,filter%5BcategoryId%5D*,filter%5BcustomAttributes%5D*,filter%5BlocationId%5D*,filter%5BsearchText%5D*,filter%5BstatusId%5D*,filter%5BstatusLabel%5D*,filter%5BupdatedAt%5D*,includeCustomAttributes*,includeDeleted*,includeSubLocations*,limit*,sort*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[AssetsRequestBuilderGetQueryParameters]] = None) -> Optional[AssetsGetResponse]:
        """
        GET assets V2
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[AssetsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .assets_get_response import AssetsGetResponse

        return await self.request_adapter.send_async(request_info, AssetsGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[AssetsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        GET assets V2
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> AssetsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: AssetsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return AssetsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class AssetsRequestBuilderGetQueryParameters():
        """
        GET assets V2
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "cursor_state":
                return "cursorState"
            if original_name == "filtercategory_id":
                return "filter%5BcategoryId%5D"
            if original_name == "filtercustom_attributes":
                return "filter%5BcustomAttributes%5D"
            if original_name == "filterlocation_id":
                return "filter%5BlocationId%5D"
            if original_name == "filtersearch_text":
                return "filter%5BsearchText%5D"
            if original_name == "filterstatus_id":
                return "filter%5BstatusId%5D"
            if original_name == "filterstatus_label":
                return "filter%5BstatusLabel%5D"
            if original_name == "filterupdated_at":
                return "filter%5BupdatedAt%5D"
            if original_name == "include_custom_attributes":
                return "includeCustomAttributes"
            if original_name == "include_deleted":
                return "includeDeleted"
            if original_name == "include_sub_locations":
                return "includeSubLocations"
            if original_name == "limit":
                return "limit"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # An opaque cursor token that identifies where the next page of paginated results should start. It's returned ineach paginated response so that it can be supplied in the next request to continue paginated results. If apaginated response contains no ``cursorState`` value, then there are no further pages to return.Omit this field to initiate a paginated request or to restart pagination.
        cursor_state: Optional[str] = None

        # An exploded array of category IDs to which all returned objects must belong. For example,``?filter[categoryId]=123&filter[categoryId]=456``.
        filtercategory_id: Optional[list[str]] = None

        # A custom attribute and value that each returned object must have. This filter is keyed by the custom attribute'sname field and set equal to the desired value. As an example, ``?filter[customAttributes][ca1]=true``. Use thisfield multiple times to specify more than one custom attribute filter.The value supplied for a custom attribute filter must match the type specified by the attribute. Values can be:- A single value, or exploded array of values, of string, boolean, or number (for ``text``, ``numeric``, ``date``, ``boolean``, ``select``, or ``multi_select`` types). Note that a string value for a ``text`` data type will perform a case-insensitive, substring match.- A range of values (inclusive) in the form: ``startValue..endValue`` (for ``numeric`` or ``date`` Types).- Partial ranges (inclusive): ``...endValue`` or ``startValue..`` (for ``numeric`` or ``date`` Types).Select and Multi-Select filters should use Custom Attribute Value IDs as values.Note that explicit value filters and range filters for a given attribute cannot be used in conjunction.Examples (without URL escaping)Example 1) Filter a ``boolean`` custom attribute:``?filter[customAttributes][ca1]=true``Example 2) Filter a ``numeric`` range custom attribute:``?filter[customAttributes][ca2]=1..5``Example 3) Filter a ``date`` custom attribute to on or before a given date:``?filter[customAttributes][ca3]=..2020-11-01``Example 4) Filter a ``select`` custom attribute:``?filter[customAttributes][ca4]=b959daad-4d00-4209-9acc-e900ac5832cf``Example 5) Filter a ``multi_select`` custom attribute to multiple values:``?filter[customAttributes][ca5]=63801bb7-db1f-49bf-9000-a392a5879f22&filter[customAttributes][ca5]=757d0934-a4a0-4af8-821d-64d611e84a56``Example 6) Filter a ``text`` custom attribute to a given input:``?filter[customAttributes][ca6]=Some text input``
        filtercustom_attributes: Optional[str] = None

        # An exploded array of location IDs. Each returned object must be associated with one of the locations specifiedby the IDs. To specify sub-locations of a location, provide a single location ID here and then set the``includeSubLocations`` field in the request to ``true``. For example,``?filter[locationId]=826e102a-36de-41e7-8c58-1b1696ccbba8&filter[locationId]=cee49807-fcc4-43ae-80a2-8ca819dfa70d``.
        filterlocation_id: Optional[list[str]] = None

        # A string that must be contained within any of a returned object's searchable text fields, including text customattributes. ``searchText`` is case-insensitive, and will match substrings as well as full strings.
        filtersearch_text: Optional[str] = None

        # An exploded array of status IDs. Each returned object must have one of the statuses specified by the IDs. Forexample, ``?filter[statusId]=84eb6a10-dde3-475f-aaf4-b5df3aebbd0b&filter[statusId]=5ba5c1af-fcd6-4506-b5e4-f20f5321dd69``.
        filterstatus_id: Optional[list[str]] = None

        # An exploded array of status labels. Each returned object must have one of the statuses bearing any of theselabels, case-insensitive. For example, ``?filter[statusLabel]=ordered&filter[statusLabel]=delivered``.
        filterstatus_label: Optional[list[str]] = None

        # A string that specifies a date and time or a date and time range at which all returned objects mast have beenupdated. A single date and time takes this format: ``YYYY-MM-DDThh:mm:ss.SSSZ``, A date and time range takes thisformat: ``YYYY-MM-DDThh:mm:ss.SSSZ..YYYY-MM-DDThh:mm:ss.SSSZ``. Range queries can be closed or open in eitherdirection: ``YYYY-MM-DDThh:mm:ss.SSSZ..`` or ``..YYYY-MM-DDThh:mm:ss.SSSZ``.
        filterupdated_at: Optional[str] = None

        # Specifies whether or not returned assets include custom attributes or not. If ``true``, they're included. If``false``, they're not. Default is ``false``.
        include_custom_attributes: Optional[bool] = None

        # Whether or not soft-deleted object should be included in the response. If ``true``, soft-deleted objects are returned. If ``false``, they are not. The default is ``false``.
        include_deleted: Optional[bool] = None

        # Specifies whether or not to consider sub-locations when filtering by ``locationId``. For this setting to work,the request must contain only a single for ``filter[locationId]``. If ``true``, the search looks for assets withinthe specified location and in all the sub-locations of the specified location. If ``false``, the search looks forassets only within the specified location(s). Default is ``false``.
        include_sub_locations: Optional[bool] = None

        # The maximum number of objects that can be returned in a page. A request might return fewer objects than the limit if the Assets service runs out of specified objects to return - at the end of a set of paged results, for example. The maximum limit is ``200``; the default limit is ``25``.
        limit: Optional[int] = None

        # A string that specifies how to sort returned objects. The string provides a valid API field name with anoptional direction, either ``asc`` (ascending) or ``desc`` (descending). In the case of custom attributes, use dotnotation to specify the attribute by name—for example, ``customAttributes.ca3 desc``. The string may containmultiple comma-separated expressions for secondary sorts. The default sort order is ``asc`` if not provided.
        sort: Optional[str] = None

    
    @dataclass
    class AssetsRequestBuilderGetRequestConfiguration(RequestConfiguration[AssetsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .status_step_sets_post_response_values import StatusStepSetsPostResponse_values

@dataclass
class StatusStepSetsPostResponse(Parsable):
    # The time when the component was created (ISO8601 Date time format in UTC).
    created_at: Optional[str] = None
    # The actor that created the component. This is an Autodesk / Oxygen ID.
    created_by: Optional[str] = None
    # The time when the component was deleted at (ISO8601 Date time format in UTC).
    deleted_at: Optional[str] = None
    # The actor that deleted the component. This is an Autodesk / Oxygen ID.
    deleted_by: Optional[str] = None
    # A description of the status set.Max length: 500
    description: Optional[str] = None
    # The ID of the component.
    id: Optional[str] = None
    # A flag indicating whether the component is active or inactive (``isActive`` is ``true`` if-and-only-if ``deletedAt`` is empty).
    is_active: Optional[bool] = None
    # Specifies if this is the status set that is assigned to categories by default - that is, this is thestatus set that will be assigned to categories if no other status set has been assigned, eitherexplicitly or through inheritance. If ``true``, it's the default status set. If ``false``, it's not.
    is_default: Optional[bool] = None
    # The display name for the status set. This name appears in the Assets UI when viewing status sets.Max length: 100
    name: Optional[str] = None
    # The Autodesk Construction Cloud project ID. Must be a UUID or a project ID of the form ``b.{UUID}``.
    project_id: Optional[UUID] = None
    # The time when the component was last updated (ISO8601 Date time format in UTC).
    updated_at: Optional[str] = None
    # The actor that last updated the component. This is an Autodesk / Oxygen ID.
    updated_by: Optional[str] = None
    # An array of statuses that belong to this status set.
    values: Optional[list[StatusStepSetsPostResponse_values]] = None
    # A global sequence number that is incremented any time a component of this type is created, updated, or deleted. If you cache components, you can use the version value to compare the cached component to the same component online to see if the component has been updated. If the online component has a higher version value, it has been updated.
    version: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> StatusStepSetsPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: StatusStepSetsPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return StatusStepSetsPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .status_step_sets_post_response_values import StatusStepSetsPostResponse_values

        from .status_step_sets_post_response_values import StatusStepSetsPostResponse_values

        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_str_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "deletedAt": lambda n : setattr(self, 'deleted_at', n.get_str_value()),
            "deletedBy": lambda n : setattr(self, 'deleted_by', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "isActive": lambda n : setattr(self, 'is_active', n.get_bool_value()),
            "isDefault": lambda n : setattr(self, 'is_default', n.get_bool_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "projectId": lambda n : setattr(self, 'project_id', n.get_uuid_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_str_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "values": lambda n : setattr(self, 'values', n.get_collection_of_object_values(StatusStepSetsPostResponse_values)),
            "version": lambda n : setattr(self, 'version', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_str_value("deletedAt", self.deleted_at)
        writer.write_str_value("deletedBy", self.deleted_by)
        writer.write_str_value("description", self.description)
        writer.write_str_value("id", self.id)
        writer.write_bool_value("isActive", self.is_active)
        writer.write_bool_value("isDefault", self.is_default)
        writer.write_str_value("name", self.name)
        writer.write_uuid_value("projectId", self.project_id)
        writer.write_str_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_collection_of_object_values("values", self.values)
        writer.write_int_value("version", self.version)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .properties_get_response import PropertiesGetResponse

class PropertiesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /cost/v1/containers/{containerId}/properties
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new PropertiesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/cost/v1/containers/{containerId}/properties{?filter%5BassociationId%5D*,filter%5BassociationType%5D*,filter%5BlastModifiedSince%5D*,filter%5Bname%5D*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[PropertiesRequestBuilderGetQueryParameters]] = None) -> Optional[PropertiesGetResponse]:
        """
        Lists all the attribute definitions created to define custom attributes for a given module.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[PropertiesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .properties_get_response import PropertiesGetResponse

        return await self.request_adapter.send_async(request_info, PropertiesGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[PropertiesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Lists all the attribute definitions created to define custom attributes for a given module.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> PropertiesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: PropertiesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return PropertiesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class PropertiesRequestBuilderGetQueryParameters():
        """
        Lists all the attribute definitions created to define custom attributes for a given module.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filterassociation_id":
                return "filter%5BassociationId%5D"
            if original_name == "filterassociation_type":
                return "filter%5BassociationType%5D"
            if original_name == "filterlast_modified_since":
                return "filter%5BlastModifiedSince%5D"
            if original_name == "filtername":
                return "filter%5Bname%5D"
            return original_name
        
        # The ID of the associated item, for example, the ID of a budget, contract, change order or cost item.Separate multiple IDs with commas, for example, ``filter[associationId]=id1,id2``.
        filterassociation_id: Optional[list[str]] = None

        # The type of the associated item.Possible values: ``Budget``, ``Contract``, ``CostItem``, ``FormDefinition``, ``Payment``, ``BudgetPayment``, ``Expense``, ``ExpenseItem``.For example, ``filter[associationType]=Budget``.
        filterassociation_type: Optional[str] = None

        # Returns only items that were modified since the specified date and time, in ISO 8601 format. For example, ``filter[lastModifiedSince]=2020-03-01T13:00:00Z``.
        filterlast_modified_since: Optional[str] = None

        # Returns only items with the specified name. For example, ``filter[name]=Labor``.Max length: 255
        filtername: Optional[str] = None

    
    @dataclass
    class PropertiesRequestBuilderGetRequestConfiguration(RequestConfiguration[PropertiesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .exports_post_request_body_options_issue_markups import ExportsPostRequestBody_options_issueMarkups
    from .exports_post_request_body_options_photo_markups import ExportsPostRequestBody_options_photoMarkups
    from .exports_post_request_body_options_standard_markups import ExportsPostRequestBody_options_standardMarkups

@dataclass
class ExportsPostRequestBody_options(Parsable):
    """
    The criteria for the markups and links to include with the exported files.Note that unpublished markups are those visible only to their creator.
    """
    # The options for which Issues markups to export.No Issues markups will be exported if both options are set to ``false``.
    issue_markups: Optional[ExportsPostRequestBody_options_issueMarkups] = None
    # The name of the file that will be produced by the export operation. This should not include the filename extension.
    output_file_name: Optional[str] = None
    # The options for which Photos markups to export.No Photos markups will be exported if both options are set to ``false``.
    photo_markups: Optional[ExportsPostRequestBody_options_photoMarkups] = None
    # The options for which standard markups and associated links to export.No markups or links will be exported if all options are set to ``false``.
    standard_markups: Optional[ExportsPostRequestBody_options_standardMarkups] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ExportsPostRequestBody_options:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ExportsPostRequestBody_options
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ExportsPostRequestBody_options()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .exports_post_request_body_options_issue_markups import ExportsPostRequestBody_options_issueMarkups
        from .exports_post_request_body_options_photo_markups import ExportsPostRequestBody_options_photoMarkups
        from .exports_post_request_body_options_standard_markups import ExportsPostRequestBody_options_standardMarkups

        from .exports_post_request_body_options_issue_markups import ExportsPostRequestBody_options_issueMarkups
        from .exports_post_request_body_options_photo_markups import ExportsPostRequestBody_options_photoMarkups
        from .exports_post_request_body_options_standard_markups import ExportsPostRequestBody_options_standardMarkups

        fields: dict[str, Callable[[Any], None]] = {
            "issueMarkups": lambda n : setattr(self, 'issue_markups', n.get_object_value(ExportsPostRequestBody_options_issueMarkups)),
            "outputFileName": lambda n : setattr(self, 'output_file_name', n.get_str_value()),
            "photoMarkups": lambda n : setattr(self, 'photo_markups', n.get_object_value(ExportsPostRequestBody_options_photoMarkups)),
            "standardMarkups": lambda n : setattr(self, 'standard_markups', n.get_object_value(ExportsPostRequestBody_options_standardMarkups)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("issueMarkups", self.issue_markups)
        writer.write_str_value("outputFileName", self.output_file_name)
        writer.write_object_value("photoMarkups", self.photo_markups)
        writer.write_object_value("standardMarkups", self.standard_markups)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_index_get_response_errors import WithIndexGetResponse_errors
    from .with_index_get_response_stats import WithIndexGetResponse_stats

@dataclass
class WithIndexGetResponse(Parsable):
    # The errors property
    errors: Optional[list[WithIndexGetResponse_errors]] = None
    # url for downloading the index fields.
    fields_url: Optional[str] = None
    # index id.
    index_id: Optional[str] = None
    # url for downloading the index manifest.
    manifest_url: Optional[str] = None
    # project id.
    project_id: Optional[str] = None
    # url for downloading the index properties.
    properties_url: Optional[str] = None
    # query id.
    query_id: Optional[str] = None
    # url for downloading the query result.
    query_results_url: Optional[str] = None
    # timestamp.
    retry_at: Optional[datetime.datetime] = None
    # unique url for this indexing job status.
    self_url: Optional[str] = None
    # job status.
    state: Optional[str] = None
    # some higher level index statistics.
    stats: Optional[WithIndexGetResponse_stats] = None
    # type.
    type: Optional[str] = None
    # timestamp.
    updated_at: Optional[datetime.datetime] = None
    # list all versions this index depends upon.
    version_urns: Optional[list[str]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithIndexGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithIndexGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithIndexGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_index_get_response_errors import WithIndexGetResponse_errors
        from .with_index_get_response_stats import WithIndexGetResponse_stats

        from .with_index_get_response_errors import WithIndexGetResponse_errors
        from .with_index_get_response_stats import WithIndexGetResponse_stats

        fields: dict[str, Callable[[Any], None]] = {
            "errors": lambda n : setattr(self, 'errors', n.get_collection_of_object_values(WithIndexGetResponse_errors)),
            "fieldsUrl": lambda n : setattr(self, 'fields_url', n.get_str_value()),
            "indexId": lambda n : setattr(self, 'index_id', n.get_str_value()),
            "manifestUrl": lambda n : setattr(self, 'manifest_url', n.get_str_value()),
            "projectId": lambda n : setattr(self, 'project_id', n.get_str_value()),
            "propertiesUrl": lambda n : setattr(self, 'properties_url', n.get_str_value()),
            "queryId": lambda n : setattr(self, 'query_id', n.get_str_value()),
            "queryResultsUrl": lambda n : setattr(self, 'query_results_url', n.get_str_value()),
            "retryAt": lambda n : setattr(self, 'retry_at', n.get_datetime_value()),
            "selfUrl": lambda n : setattr(self, 'self_url', n.get_str_value()),
            "state": lambda n : setattr(self, 'state', n.get_str_value()),
            "stats": lambda n : setattr(self, 'stats', n.get_object_value(WithIndexGetResponse_stats)),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "versionUrns": lambda n : setattr(self, 'version_urns', n.get_collection_of_primitive_values(str)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("errors", self.errors)
        writer.write_str_value("fieldsUrl", self.fields_url)
        writer.write_str_value("indexId", self.index_id)
        writer.write_str_value("manifestUrl", self.manifest_url)
        writer.write_str_value("projectId", self.project_id)
        writer.write_str_value("propertiesUrl", self.properties_url)
        writer.write_str_value("queryId", self.query_id)
        writer.write_str_value("queryResultsUrl", self.query_results_url)
        writer.write_datetime_value("retryAt", self.retry_at)
        writer.write_str_value("selfUrl", self.self_url)
        writer.write_str_value("state", self.state)
        writer.write_object_value("stats", self.stats)
        writer.write_str_value("type", self.type)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_collection_of_primitive_values("versionUrns", self.version_urns)
    




from enum import Enum

class ProjectsPostResponse_products_status(str, Enum):
    Activating = "activating",
    ActivationFailed = "activationFailed",
    Active = "active",
    Deactivating = "deactivating",
    DeactivationFailed = "deactivationFailed",
    Inactive = "inactive",
    Available = "available",




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .classification_systems.classification_systems_request_builder import ClassificationSystemsRequestBuilder
    from .content_views.content_views_request_builder import ContentViewsRequestBuilder
    from .packages.packages_request_builder import PackagesRequestBuilder
    from .settings.settings_request_builder import SettingsRequestBuilder

class WithProjectItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/takeoff/v1/projects/{projectId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithProjectItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/takeoff/v1/projects/{projectId}", path_parameters)
    
    @property
    def classification_systems(self) -> ClassificationSystemsRequestBuilder:
        """
        The classificationSystems property
        """
        from .classification_systems.classification_systems_request_builder import ClassificationSystemsRequestBuilder

        return ClassificationSystemsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def content_views(self) -> ContentViewsRequestBuilder:
        """
        The contentViews property
        """
        from .content_views.content_views_request_builder import ContentViewsRequestBuilder

        return ContentViewsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def packages(self) -> PackagesRequestBuilder:
        """
        The packages property
        """
        from .packages.packages_request_builder import PackagesRequestBuilder

        return PackagesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def settings(self) -> SettingsRequestBuilder:
        """
        The settings property
        """
        from .settings.settings_request_builder import SettingsRequestBuilder

        return SettingsRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .templates_get_response_results_steps_tasks import TemplatesGetResponse_results_steps_tasks

@dataclass
class TemplatesGetResponse_results_steps(Parsable):
    # Specifies a dynamic due date. When the step starts, the due date is calculated based on this field.
    days_to_respond: Optional[float] = None
    # The internal, globally unique identifier (UUID) for the step. Each step in the workflow has a unique ID.
    id: Optional[UUID] = None
    # The number representing the order of the steps, where ``1`` is the first step.
    step_number: Optional[float] = None
    # A list of tasks associated with this template. Each task has its own unique identifier (UUID).
    tasks: Optional[list[TemplatesGetResponse_results_steps_tasks]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> TemplatesGetResponse_results_steps:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: TemplatesGetResponse_results_steps
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return TemplatesGetResponse_results_steps()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .templates_get_response_results_steps_tasks import TemplatesGetResponse_results_steps_tasks

        from .templates_get_response_results_steps_tasks import TemplatesGetResponse_results_steps_tasks

        fields: dict[str, Callable[[Any], None]] = {
            "daysToRespond": lambda n : setattr(self, 'days_to_respond', n.get_float_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "stepNumber": lambda n : setattr(self, 'step_number', n.get_float_value()),
            "tasks": lambda n : setattr(self, 'tasks', n.get_collection_of_object_values(TemplatesGetResponse_results_steps_tasks)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_float_value("daysToRespond", self.days_to_respond)
        writer.write_uuid_value("id", self.id)
        writer.write_float_value("stepNumber", self.step_number)
        writer.write_collection_of_object_values("tasks", self.tasks)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .users_post_response_phone_phone_type import UsersPostResponse_phone_phoneType

@dataclass
class UsersPostResponse_phone(Parsable):
    """
    The user's phone number. This data syncs from the user's Autodesk profile.
    """
    # User's phone extension.
    extension: Optional[str] = None
    # User's phone number
    number: Optional[str] = None
    # The user's phone type.Possible values: ``home``, ``mobile``, or ``office``. Default value: ``mobile``.
    phone_type: Optional[UsersPostResponse_phone_phoneType] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> UsersPostResponse_phone:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: UsersPostResponse_phone
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return UsersPostResponse_phone()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .users_post_response_phone_phone_type import UsersPostResponse_phone_phoneType

        from .users_post_response_phone_phone_type import UsersPostResponse_phone_phoneType

        fields: dict[str, Callable[[Any], None]] = {
            "extension": lambda n : setattr(self, 'extension', n.get_str_value()),
            "number": lambda n : setattr(self, 'number', n.get_str_value()),
            "phoneType": lambda n : setattr(self, 'phone_type', n.get_enum_value(UsersPostResponse_phone_phoneType)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("extension", self.extension)
        writer.write_str_value("number", self.number)
        writer.write_enum_value("phoneType", self.phone_type)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_version_set_patch_response_collection import WithVersionSetPatchResponse_collection

@dataclass
class WithVersionSetPatchResponse(Parsable):
    # The collection object, if assigned. If no collection is assigned, this value is ``null``.
    collection: Optional[WithVersionSetPatchResponse_collection] = None
    # The time when the version set was created, in ISO-8601 format (YYYY-MM-DDTHH:mm:ss.SSSZ).
    created_at: Optional[datetime.datetime] = None
    # The ID of the user who created the version set.
    created_by: Optional[str] = None
    # The name of the user who created the version set.
    created_by_name: Optional[str] = None
    # The ID of the version set.
    id: Optional[UUID] = None
    # The issuance date of the version set, in ISO-8601 date format (YYYY-MM-DD).
    issuance_date: Optional[datetime.datetime] = None
    # The name of the version set.
    name: Optional[str] = None
    # The time when the version set was last updated, in ISO-8601 format (YYYY-MM-DDTHH:mm:ss.SSSZ).
    updated_at: Optional[datetime.datetime] = None
    # The ID of the user who last updated the version set.
    updated_by: Optional[str] = None
    # The name of the user who last updated the version set.
    updated_by_name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithVersionSetPatchResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithVersionSetPatchResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithVersionSetPatchResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_version_set_patch_response_collection import WithVersionSetPatchResponse_collection

        from .with_version_set_patch_response_collection import WithVersionSetPatchResponse_collection

        fields: dict[str, Callable[[Any], None]] = {
            "collection": lambda n : setattr(self, 'collection', n.get_object_value(WithVersionSetPatchResponse_collection)),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "createdByName": lambda n : setattr(self, 'created_by_name', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "issuanceDate": lambda n : setattr(self, 'issuance_date', n.get_datetime_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "updatedByName": lambda n : setattr(self, 'updated_by_name', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("collection", self.collection)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_str_value("createdByName", self.created_by_name)
        writer.write_uuid_value("id", self.id)
        writer.write_datetime_value("issuanceDate", self.issuance_date)
        writer.write_str_value("name", self.name)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_str_value("updatedByName", self.updated_by_name)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .linked_files_get_response_host_file_publish_status import LinkedFilesGetResponse_hostFile_publishStatus

@dataclass
class LinkedFilesGetResponse_hostFile(Parsable):
    """
    Metadata about the host model (the main Revit model that contains linked files). This object is only returned in the response when ``includeHost`` is set to ``true``.
    """
    # The URN of the host model.
    item_id: Optional[str] = None
    # The name of the host model.
    model_name: Optional[str] = None
    # Indicates whether the host model has been published. Possible values:``published`` – The model has been published.``notPublished`` – The model has not been published.
    publish_status: Optional[LinkedFilesGetResponse_hostFile_publishStatus] = None
    # A temporary URL used to download the host model. Signed URLs are valid for 1 hour.
    signed_url: Optional[str] = None
    # The file size of the host model in bytes.
    size: Optional[int] = None
    # The version ID of the host model.
    version_id: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> LinkedFilesGetResponse_hostFile:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: LinkedFilesGetResponse_hostFile
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return LinkedFilesGetResponse_hostFile()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .linked_files_get_response_host_file_publish_status import LinkedFilesGetResponse_hostFile_publishStatus

        from .linked_files_get_response_host_file_publish_status import LinkedFilesGetResponse_hostFile_publishStatus

        fields: dict[str, Callable[[Any], None]] = {
            "itemId": lambda n : setattr(self, 'item_id', n.get_str_value()),
            "modelName": lambda n : setattr(self, 'model_name', n.get_str_value()),
            "publishStatus": lambda n : setattr(self, 'publish_status', n.get_enum_value(LinkedFilesGetResponse_hostFile_publishStatus)),
            "signedUrl": lambda n : setattr(self, 'signed_url', n.get_str_value()),
            "size": lambda n : setattr(self, 'size', n.get_int_value()),
            "versionId": lambda n : setattr(self, 'version_id', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("itemId", self.item_id)
        writer.write_str_value("modelName", self.model_name)
        writer.write_enum_value("publishStatus", self.publish_status)
        writer.write_str_value("signedUrl", self.signed_url)
        writer.write_int_value("size", self.size)
        writer.write_str_value("versionId", self.version_id)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .attachments_post_response_async_state import AttachmentsPostResponse_asyncState
    from .attachments_post_response_category_id import AttachmentsPostResponse_categoryId
    from .attachments_post_response_permitted_actions import AttachmentsPostResponse_permittedActions
    from .attachments_post_response_urn_type_id import AttachmentsPostResponse_urnTypeId

@dataclass
class AttachmentsPostResponse(Parsable):
    # Represents the state of the asynchronous process triggered after marking the file upload as complete (``isFileUploaded=true``). The backend initiates this process to generate the ``URN`` for the attachment.Possible values:- ``1`` - Pending (the request is queued and awaiting processing).- ``2`` - Started (the backend process to generate the ``URN`` has begun).- ``3`` - Success (the ``URN`` was successfully created).- ``4`` - Failure (the process failed, and the ``URN`` was not created).
    async_state: Optional[AttachmentsPostResponse_asyncState] = None
    # The workflow state of the submittal item associated with the attachment.- ``Category IDs 1-4``: These represent active workflow states and can be set when creating an attachment.- ``Category IDs 5-8``: These represent historical records of attachments from earlier revisions of the submittal process and are automatically assigned by the system.Possible values:- ``1`` (Submission)- ``2`` (For Review)- ``3`` (Review Response)- ``4`` (Final Response)- ``5`` (Previous Submission)- ``6`` (Previous For Review)- ``7`` (Previous Review Response)- ``8`` (Previous Final Response)
    category_id: Optional[AttachmentsPostResponse_categoryId] = None
    # The date and time when the attachment created, in ISO 8601 format. For example, ``2018-02-01T12:09:24.198466Z``.
    created_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who created the attachment.
    created_by: Optional[str] = None
    # The UUID of the source file the attachment was duplicated from. This indicates that the attachment was created as a copy of an existing file, preserving its metadata and association with the original file.
    duplicated_from: Optional[UUID] = None
    # The internal, globally unique identifier (UUID) for the attachment.
    id: Optional[UUID] = None
    # Indicates whether the attachment upload is complete.``true``: the attachment upload is complete.``false``: the upload is still in progress or pending completion.
    is_file_uploaded: Optional[bool] = None
    # The ID of the submittal item associated with the attachment.
    item_id: Optional[UUID] = None
    # The user-defined name of the attachment. This value is specified when the attachment is created and may differ from the original file name.
    name: Optional[str] = None
    # A list of actions that the user is allowed to perform on the attachment.
    permitted_actions: Optional[list[AttachmentsPostResponse_permittedActions]] = None
    # Not relevant
    resource_urns: Optional[str] = None
    # Not relevant
    revision: Optional[int] = None
    # Not relevant
    revision_folder_urn: Optional[str] = None
    # The ID of the task within the review step to which the attachment is associated.- If the attachment was added to a review step, this field contains the task ID that identifies the relevant task.- If the attachment was not part of a review, this field is ``null``.
    task_id: Optional[UUID] = None
    # The date and time the attachment was last updated, in ISO 8601 format. For example, ``2018-02-01T12:09:24.198466Z``.
    updated_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who last updated the attachment.
    updated_by: Optional[str] = None
    # The unique identifier for the upload session associated with the attachment applicable only to local file uploads. This value is used to generate a URL for uploading the actual file.
    upload_urn: Optional[str] = None
    # Not relevant
    url: Optional[str] = None
    # The unique identifier for a specific file version in the Files tool.
    urn: Optional[str] = None
    # Not relevant
    urn_page: Optional[str] = None
    # Specifies the type of urn associated with the attachment. This value identifies the storage type for the file reference.Possible values: ``2``This value is always set to ``2`` for both local and Files tool attachments.
    urn_type_id: Optional[AttachmentsPostResponse_urnTypeId] = None
    # The version number of the file in Autodesk Docs.
    urn_version: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AttachmentsPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AttachmentsPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AttachmentsPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .attachments_post_response_async_state import AttachmentsPostResponse_asyncState
        from .attachments_post_response_category_id import AttachmentsPostResponse_categoryId
        from .attachments_post_response_permitted_actions import AttachmentsPostResponse_permittedActions
        from .attachments_post_response_urn_type_id import AttachmentsPostResponse_urnTypeId

        from .attachments_post_response_async_state import AttachmentsPostResponse_asyncState
        from .attachments_post_response_category_id import AttachmentsPostResponse_categoryId
        from .attachments_post_response_permitted_actions import AttachmentsPostResponse_permittedActions
        from .attachments_post_response_urn_type_id import AttachmentsPostResponse_urnTypeId

        fields: dict[str, Callable[[Any], None]] = {
            "asyncState": lambda n : setattr(self, 'async_state', n.get_enum_value(AttachmentsPostResponse_asyncState)),
            "categoryId": lambda n : setattr(self, 'category_id', n.get_enum_value(AttachmentsPostResponse_categoryId)),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "duplicatedFrom": lambda n : setattr(self, 'duplicated_from', n.get_uuid_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "isFileUploaded": lambda n : setattr(self, 'is_file_uploaded', n.get_bool_value()),
            "itemId": lambda n : setattr(self, 'item_id', n.get_uuid_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "permittedActions": lambda n : setattr(self, 'permitted_actions', n.get_collection_of_object_values(AttachmentsPostResponse_permittedActions)),
            "resourceUrns": lambda n : setattr(self, 'resource_urns', n.get_str_value()),
            "revision": lambda n : setattr(self, 'revision', n.get_int_value()),
            "revisionFolderUrn": lambda n : setattr(self, 'revision_folder_urn', n.get_str_value()),
            "taskId": lambda n : setattr(self, 'task_id', n.get_uuid_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "uploadUrn": lambda n : setattr(self, 'upload_urn', n.get_str_value()),
            "url": lambda n : setattr(self, 'url', n.get_str_value()),
            "urn": lambda n : setattr(self, 'urn', n.get_str_value()),
            "urnPage": lambda n : setattr(self, 'urn_page', n.get_str_value()),
            "urnTypeId": lambda n : setattr(self, 'urn_type_id', n.get_enum_value(AttachmentsPostResponse_urnTypeId)),
            "urnVersion": lambda n : setattr(self, 'urn_version', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_enum_value("asyncState", self.async_state)
        writer.write_enum_value("categoryId", self.category_id)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_uuid_value("duplicatedFrom", self.duplicated_from)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("isFileUploaded", self.is_file_uploaded)
        writer.write_uuid_value("itemId", self.item_id)
        writer.write_str_value("name", self.name)
        writer.write_collection_of_object_values("permittedActions", self.permitted_actions)
        writer.write_str_value("resourceUrns", self.resource_urns)
        writer.write_int_value("revision", self.revision)
        writer.write_str_value("revisionFolderUrn", self.revision_folder_urn)
        writer.write_uuid_value("taskId", self.task_id)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_str_value("uploadUrn", self.upload_urn)
        writer.write_str_value("url", self.url)
        writer.write_str_value("urn", self.urn)
        writer.write_str_value("urnPage", self.urn_page)
        writer.write_enum_value("urnTypeId", self.urn_type_id)
        writer.write_int_value("urnVersion", self.urn_version)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .form_templates_get_response_data_forms import FormTemplatesGetResponse_data_forms

@dataclass
class FormTemplatesGetResponse_data(Parsable):
    # The unique identifier of the user who created the template.
    created_by: Optional[str] = None
    # Reference to fetch forms created from this template.
    forms: Optional[FormTemplatesGetResponse_data_forms] = None
    # Permissions on this template assigned to companies and roles.
    group_permissions: Optional[list[str]] = None
    # The unique identifier of the template.
    id: Optional[str] = None
    # A flag that indicates whether the template has a PDF or not.
    is_pdf: Optional[bool] = None
    # Display name of template.
    name: Optional[str] = None
    # For PDF forms, the URL to download the form's PDF.
    pdf_url: Optional[str] = None
    # Unique indentifier of the project the template belongs to.
    project_id: Optional[str] = None
    # Status of template: ````active````, ````inactive```` (archived), or ````deleted````
    status: Optional[str] = None
    # User supplied type of template
    template_type: Optional[str] = None
    # When the template was last updated, UTC date and time in ISO-8601 format.
    updated_at: Optional[datetime.datetime] = None
    # Permissions on this template assigned to individual users.
    user_permissions: Optional[list[str]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> FormTemplatesGetResponse_data:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: FormTemplatesGetResponse_data
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return FormTemplatesGetResponse_data()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .form_templates_get_response_data_forms import FormTemplatesGetResponse_data_forms

        from .form_templates_get_response_data_forms import FormTemplatesGetResponse_data_forms

        fields: dict[str, Callable[[Any], None]] = {
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "forms": lambda n : setattr(self, 'forms', n.get_object_value(FormTemplatesGetResponse_data_forms)),
            "groupPermissions": lambda n : setattr(self, 'group_permissions', n.get_collection_of_primitive_values(str)),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "isPdf": lambda n : setattr(self, 'is_pdf', n.get_bool_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "pdfUrl": lambda n : setattr(self, 'pdf_url', n.get_str_value()),
            "projectId": lambda n : setattr(self, 'project_id', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_str_value()),
            "templateType": lambda n : setattr(self, 'template_type', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "userPermissions": lambda n : setattr(self, 'user_permissions', n.get_collection_of_primitive_values(str)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("createdBy", self.created_by)
        writer.write_object_value("forms", self.forms)
        writer.write_collection_of_primitive_values("groupPermissions", self.group_permissions)
        writer.write_str_value("id", self.id)
        writer.write_bool_value("isPdf", self.is_pdf)
        writer.write_str_value("name", self.name)
        writer.write_str_value("pdfUrl", self.pdf_url)
        writer.write_str_value("projectId", self.project_id)
        writer.write_str_value("status", self.status)
        writer.write_str_value("templateType", self.template_type)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_collection_of_primitive_values("userPermissions", self.user_permissions)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ExportsPostRequestBody_options_standardMarkups(Parsable):
    """
    The list of types of standard markups and their associated links that you want to include in the export.By default markups will be exported and hyperlinks will not be exported.
    """
    # ``true``: include links in the markups. Supported links are links to Sheets, Files, RFIs, Forms, Submittals, and Assets.``false``: (default) do not include links in markups.Note that if both ``includePublishedMarkups`` and ``includeUnpublishedMarkups`` are set to ``false``, this parameter will be treated as ``false``.
    include_markup_links: Optional[bool] = None
    # ``true``: (default) include all published standard markups in the exported sheets. Note that published markups are visible to all project members.``false``: export sheets without including published standard markups.
    include_published_markups: Optional[bool] = None
    # ``true``: (default) standard markups that are visible only to their creators will be included in the exported sheets.``false``: exported sheets will exclude unpublished markups.
    include_unpublished_markups: Optional[bool] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ExportsPostRequestBody_options_standardMarkups:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ExportsPostRequestBody_options_standardMarkups
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ExportsPostRequestBody_options_standardMarkups()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "includeMarkupLinks": lambda n : setattr(self, 'include_markup_links', n.get_bool_value()),
            "includePublishedMarkups": lambda n : setattr(self, 'include_published_markups', n.get_bool_value()),
            "includeUnpublishedMarkups": lambda n : setattr(self, 'include_unpublished_markups', n.get_bool_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("includeMarkupLinks", self.include_markup_links)
        writer.write_bool_value("includePublishedMarkups", self.include_published_markups)
        writer.write_bool_value("includeUnpublishedMarkups", self.include_unpublished_markups)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .attachments_post_request_body_category_id import AttachmentsPostRequestBody_categoryId
    from .attachments_post_request_body_urn_type_id import AttachmentsPostRequestBody_urnTypeId

@dataclass
class AttachmentsPostRequestBody(Parsable):
    # Specifies the workflow state of the submittal item associated with the attachment. If omitted, the system assigns one based on the current state.Note that you can only set category IDs ``1``-``4`` when adding an attachment. Category IDs ``5``-``8`` are system-generated and appear in responses for historical records.In some cases, you must manually set ``categoryId``:1. Subcontractor Submissions:When a subcontractor creates an item in the ``Open (Submitted) - (mgr-1)`` state,  set ``categoryId`` to ``1`` (**Submission**) instead of ``2`` (**For Review**).2. Final Response in Non-Standard Closures:When closing an item from a state other than ``Open (Reviewed) - (mgr-2)``, set ``categoryId`` to ``4`` (**Final Response**) if uploading a final response.Possible values:- ``1``: **Submission** – Initial submission.- ``2``: **For Review** – Submitted and awaiting review.- ``3``: **Review Response** – A response provided during review.- ``4``: **Final Response** – The final response in the submittal workflow.To retrieve the list of available category ID values, call `GET metadata </en/docs/acc/v1/reference/http/submittals-metadata-GET/>`_ and refer to the attachment categories list in the response.
    category_id: Optional[AttachmentsPostRequestBody_categoryId] = None
    # Specifies whether the attachment upload has been completed. This value controls how the system processes the attachment.- ``true``: if the attachment is complete.- ``false``: if the attachment is still in progress.Behavior:- For local file uploads: The system initially considers the file as not uploaded (``false``). After completing the upload, you must set ``isFileUploaded: true`` using `PATCH attachments/:id </en/docs/acc/v1/reference/http/submittals-attachments-attachmentId-PATCH/>`_ to allow the system to process the attachment.- For Files tool attachments: Always set to ``true`` since the file was uploaded earlier.This field cannot be explicitly set to ``false`` in the request. If omitted, it defaults to ``false``.
    is_file_uploaded: Optional[bool] = None
    # The user-defined name for the attachment. This value does not need to match the actual file name. For example, ``design-review.pdf`` or ``Design Review``.- Spaces are allowed in the name.- The following characters are not supported: ``/``, ``n``, ``r``, ``t``, ``0``, ``f``, ``?``, ``*``, ````, ``<``, ``>``, ``|``, ``:``.
    name: Optional[str] = None
    # The ID of the task within the review step that the attachment is associated with. The task ID identifies the specific task in the review process where the attachment belongs.This field is only required when the submittal item is in the **Review (``rev``)** state.To retrieve the task ID, call `Get tasks </en/docs/acc/v1/reference/http/submittals-tasks-GET/>`_.
    task_id: Optional[UUID] = None
    # The unique identifier for the file version in the Files tool. This field is required for referencing a specific file version when attaching a file from the Files tool.This field is not applicable for local file uploads.When attaching a file from the Files tool, both ``urn`` and ``urnTypeId`` must be provided.
    urn: Optional[str] = None
    # Specifies the type of URN associated with the attachment.Possible values: ``2``
    urn_type_id: Optional[AttachmentsPostRequestBody_urnTypeId] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AttachmentsPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AttachmentsPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AttachmentsPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .attachments_post_request_body_category_id import AttachmentsPostRequestBody_categoryId
        from .attachments_post_request_body_urn_type_id import AttachmentsPostRequestBody_urnTypeId

        from .attachments_post_request_body_category_id import AttachmentsPostRequestBody_categoryId
        from .attachments_post_request_body_urn_type_id import AttachmentsPostRequestBody_urnTypeId

        fields: dict[str, Callable[[Any], None]] = {
            "categoryId": lambda n : setattr(self, 'category_id', n.get_enum_value(AttachmentsPostRequestBody_categoryId)),
            "isFileUploaded": lambda n : setattr(self, 'is_file_uploaded', n.get_bool_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "taskId": lambda n : setattr(self, 'task_id', n.get_uuid_value()),
            "urn": lambda n : setattr(self, 'urn', n.get_str_value()),
            "urnTypeId": lambda n : setattr(self, 'urn_type_id', n.get_enum_value(AttachmentsPostRequestBody_urnTypeId)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_enum_value("categoryId", self.category_id)
        writer.write_bool_value("isFileUploaded", self.is_file_uploaded)
        writer.write_str_value("name", self.name)
        writer.write_uuid_value("taskId", self.task_id)
        writer.write_str_value("urn", self.urn)
        writer.write_enum_value("urnTypeId", self.urn_type_id)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_budget_patch_request_body_segment_code_map import WithBudgetPatchRequestBody_segmentCodeMap

@dataclass
class WithBudgetPatchRequestBody(Parsable):
    # The total amount of actual cost of the budget.
    actual_cost: Optional[float] = None
    # Not relevant
    actual_end_date: Optional[str] = None
    # Actual quantity of labor, material,... planned for a budget.
    actual_quantity: Optional[float] = None
    # Not relevant
    actual_start_date: Optional[str] = None
    # Actual unit price of a budget.
    actual_unit_price: Optional[float] = None
    # Unique code compliant with the budget code template defined by the project admin. Ignored if ``segmentCodeMap`` is defined.Max length: 255
    code: Optional[str] = None
    # Detailed description of the budget.
    description: Optional[str] = None
    # Not relevant
    duration_days: Optional[float] = None
    # The identifier assigned to an item in its original external ERP system. Use this ID to track and look up data within the integrated system. Note that this value comes from the item's ID in the external system.Max length: 255
    external_id: Optional[str] = None
    # A message generated by the external ERP system that explains the sync status of the integration. For example, common values include ``success`` or ``fail`` to indicate the result of the integration operation.Max length: 255
    external_message: Optional[str] = None
    # The name of the external ERP system integrated with Cost Management. Use this name to identify and search for data within the integrated system.Max length: 255
    external_system: Optional[str] = None
    # The input quantity planned for the budget.
    input_quantity: Optional[float] = None
    # The state of the item during the integration with the external ERP system (such as SignNow). An item can be a ``budget``, ``contract``, ``main contract``, ``main contract item``, ``cost item``, ``expense``, ``expense item``, ``change order``, or ``schedule of value``. For more details, see `Integrate with External System </en/docs/acc/v1/tutorials/cost/integrate-with-external-system/>`_ tutorial.Possible values:``locked``: the item is currently locked within the ERP system, preventing modifications until unlocked. To unlock and modify the item, use the relevant PATCH endpoint to set ``integrationState`` to ``null``. For example, for a budget, call `PATCH budgets <en/docs/bim360/v1/reference/http/cost-budgets-budgetId-PATCH/>`_. For a contract, call `PATCH contracts <en/docs/bim360/v1/reference/http/cost-contracts-contractId-PATCH/>`_. For more details, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Integrated_and_Locked>`_.``integrated``: the item has been successfully added to the ERP system.``failed``: the item encountered an error during the integration process and was not successfully added to the ERP system. For example, if a user tries to integrate ``contracts`` from an ERP system and the updates fail, the ``integrationState`` can be set to ``failed``. Retry the sync process or analyze the issue if it continues to fail.``null``: The item has not been integrated with the ERP system. This is default value.For more information regarding integrations within the Cost Management system, see `Integrations in Cost Management <https://help.autodesk.com/view/BUILD/ENU/?guid=Cost_Integrations>`_.
    integration_state: Optional[str] = None
    # Not relevant
    is_markup: Optional[bool] = None
    # The date and time when the item was last synchronized with the external ERP system. This value is updated by the external system and is in ISO 8601 format.
    last_sync_time: Optional[datetime.datetime] = None
    # A list of the IDs of the project locations where this item applies.For more information, see the Locations `Help documentation <https://aps.autodesk.com/en/docs/bim360/v1/reference/http/locations-nodes-GET/>`_ help.
    locations: Optional[list[str]] = None
    # The locked budget item field. You can lock the budget item's amount (``originalAmount``), quantity (``quantity``), or unit cost (``unitPrice``) when calculating a budget.
    locked_field: Optional[str] = None
    # Not relevant
    main_contract_amount_source: Optional[str] = None
    # Name of the budget.Max length: 1024
    name: Optional[str] = None
    # Not relevant
    original_cost_unit_price: Optional[float] = None
    # Not relevant
    planned_end_date: Optional[str] = None
    # Not relevant
    planned_start_date: Optional[str] = None
    # The quantity of labor, material, and other items planned for the budget.
    quantity: Optional[float] = None
    # Map of budget code segments to be used in the budget code, required when updating root budget code with variable length segment. Key is the budget code segment ID, value is the code for the segment.
    segment_code_map: Optional[WithBudgetPatchRequestBody_segmentCodeMap] = None
    # Unit of measures used in the budget.
    unit: Optional[str] = None
    # Unit price of a budget.
    unit_price: Optional[float] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithBudgetPatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithBudgetPatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithBudgetPatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_budget_patch_request_body_segment_code_map import WithBudgetPatchRequestBody_segmentCodeMap

        from .with_budget_patch_request_body_segment_code_map import WithBudgetPatchRequestBody_segmentCodeMap

        fields: dict[str, Callable[[Any], None]] = {
            "actualCost": lambda n : setattr(self, 'actual_cost', n.get_float_value()),
            "actualEndDate": lambda n : setattr(self, 'actual_end_date', n.get_str_value()),
            "actualQuantity": lambda n : setattr(self, 'actual_quantity', n.get_float_value()),
            "actualStartDate": lambda n : setattr(self, 'actual_start_date', n.get_str_value()),
            "actualUnitPrice": lambda n : setattr(self, 'actual_unit_price', n.get_float_value()),
            "code": lambda n : setattr(self, 'code', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "durationDays": lambda n : setattr(self, 'duration_days', n.get_float_value()),
            "externalId": lambda n : setattr(self, 'external_id', n.get_str_value()),
            "externalMessage": lambda n : setattr(self, 'external_message', n.get_str_value()),
            "externalSystem": lambda n : setattr(self, 'external_system', n.get_str_value()),
            "inputQuantity": lambda n : setattr(self, 'input_quantity', n.get_float_value()),
            "integrationState": lambda n : setattr(self, 'integration_state', n.get_str_value()),
            "isMarkup": lambda n : setattr(self, 'is_markup', n.get_bool_value()),
            "lastSyncTime": lambda n : setattr(self, 'last_sync_time', n.get_datetime_value()),
            "locations": lambda n : setattr(self, 'locations', n.get_collection_of_primitive_values(str)),
            "lockedField": lambda n : setattr(self, 'locked_field', n.get_str_value()),
            "mainContractAmountSource": lambda n : setattr(self, 'main_contract_amount_source', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "originalCostUnitPrice": lambda n : setattr(self, 'original_cost_unit_price', n.get_float_value()),
            "plannedEndDate": lambda n : setattr(self, 'planned_end_date', n.get_str_value()),
            "plannedStartDate": lambda n : setattr(self, 'planned_start_date', n.get_str_value()),
            "quantity": lambda n : setattr(self, 'quantity', n.get_float_value()),
            "segmentCodeMap": lambda n : setattr(self, 'segment_code_map', n.get_object_value(WithBudgetPatchRequestBody_segmentCodeMap)),
            "unit": lambda n : setattr(self, 'unit', n.get_str_value()),
            "unitPrice": lambda n : setattr(self, 'unit_price', n.get_float_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_float_value("actualCost", self.actual_cost)
        writer.write_str_value("actualEndDate", self.actual_end_date)
        writer.write_float_value("actualQuantity", self.actual_quantity)
        writer.write_str_value("actualStartDate", self.actual_start_date)
        writer.write_float_value("actualUnitPrice", self.actual_unit_price)
        writer.write_str_value("code", self.code)
        writer.write_str_value("description", self.description)
        writer.write_float_value("durationDays", self.duration_days)
        writer.write_str_value("externalId", self.external_id)
        writer.write_str_value("externalMessage", self.external_message)
        writer.write_str_value("externalSystem", self.external_system)
        writer.write_float_value("inputQuantity", self.input_quantity)
        writer.write_str_value("integrationState", self.integration_state)
        writer.write_bool_value("isMarkup", self.is_markup)
        writer.write_datetime_value("lastSyncTime", self.last_sync_time)
        writer.write_collection_of_primitive_values("locations", self.locations)
        writer.write_str_value("lockedField", self.locked_field)
        writer.write_str_value("mainContractAmountSource", self.main_contract_amount_source)
        writer.write_str_value("name", self.name)
        writer.write_float_value("originalCostUnitPrice", self.original_cost_unit_price)
        writer.write_str_value("plannedEndDate", self.planned_end_date)
        writer.write_str_value("plannedStartDate", self.planned_start_date)
        writer.write_float_value("quantity", self.quantity)
        writer.write_object_value("segmentCodeMap", self.segment_code_map)
        writer.write_str_value("unit", self.unit)
        writer.write_float_value("unitPrice", self.unit_price)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ManifestGetResponse_seedFiles_views(Parsable):
    # view ID.
    id: Optional[str] = None
    # is this a 3D view?.
    is3d: Optional[bool] = None
    # file version URN.
    urn: Optional[str] = None
    # viewable GUID.
    viewable_guid: Optional[str] = None
    # viewable ID.
    viewable_id: Optional[str] = None
    # viewable name.
    viewable_name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ManifestGetResponse_seedFiles_views:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ManifestGetResponse_seedFiles_views
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ManifestGetResponse_seedFiles_views()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "is3d": lambda n : setattr(self, 'is3d', n.get_bool_value()),
            "urn": lambda n : setattr(self, 'urn', n.get_str_value()),
            "viewableGuid": lambda n : setattr(self, 'viewable_guid', n.get_str_value()),
            "viewableId": lambda n : setattr(self, 'viewable_id', n.get_str_value()),
            "viewableName": lambda n : setattr(self, 'viewable_name', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("id", self.id)
        writer.write_bool_value("is3d", self.is3d)
        writer.write_str_value("urn", self.urn)
        writer.write_str_value("viewableGuid", self.viewable_guid)
        writer.write_str_value("viewableId", self.viewable_id)
        writer.write_str_value("viewableName", self.viewable_name)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class PerformanceTrackingItemsGetResponse(Parsable):
    # The ID of the tracking item's underlying budget.
    budget_id: Optional[UUID] = None
    # The code of the tracking item's underlying budget.
    code: Optional[str] = None
    # The ID of the cost container for the project to which this tracking item belongs.
    container_id: Optional[UUID] = None
    # The description of the tracking item's underlying budget.
    description: Optional[str] = None
    # The ID of the tracking item.
    id: Optional[UUID] = None
    # A list of the IDs of the project locations where this item applies, along with the node paths of these locations in the project's locations tree.For more information, see the Locations `Help documentation <https://aps.autodesk.com/en/docs/bim360/v1/reference/http/locations-nodes-GET/>`_ help.
    location_paths: Optional[list[str]] = None
    # A list of the IDs of the project locations where this item applies.For more information, see the Locations `Help documentation <https://aps.autodesk.com/en/docs/bim360/v1/reference/http/locations-nodes-GET/>`_ help.
    locations: Optional[list[str]] = None
    # The name of the tracking item's underlying budget.
    name: Optional[str] = None
    # Original amount of the budget, equals to ``quantity`` * ``unitPrice``
    original_amount: Optional[float] = None
    # The planned end date of the performance tracking item's underlying budget.
    planned_end_date: Optional[datetime.datetime] = None
    # The planned start date of the performance tracking item's underlying budget.
    planned_start_date: Optional[datetime.datetime] = None
    # The quantity of the performance tracking item's underlying budget.
    quantity: Optional[float] = None
    # The unit of the performance tracking item's underlying budget.
    unit: Optional[str] = None
    # The unit price of the performance tracking item's underlying budget.
    unit_price: Optional[float] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PerformanceTrackingItemsGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PerformanceTrackingItemsGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PerformanceTrackingItemsGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "budgetId": lambda n : setattr(self, 'budget_id', n.get_uuid_value()),
            "code": lambda n : setattr(self, 'code', n.get_str_value()),
            "containerId": lambda n : setattr(self, 'container_id', n.get_uuid_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "locationPaths": lambda n : setattr(self, 'location_paths', n.get_collection_of_primitive_values(str)),
            "locations": lambda n : setattr(self, 'locations', n.get_collection_of_primitive_values(str)),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "originalAmount": lambda n : setattr(self, 'original_amount', n.get_float_value()),
            "plannedEndDate": lambda n : setattr(self, 'planned_end_date', n.get_datetime_value()),
            "plannedStartDate": lambda n : setattr(self, 'planned_start_date', n.get_datetime_value()),
            "quantity": lambda n : setattr(self, 'quantity', n.get_float_value()),
            "unit": lambda n : setattr(self, 'unit', n.get_str_value()),
            "unitPrice": lambda n : setattr(self, 'unit_price', n.get_float_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("budgetId", self.budget_id)
        writer.write_str_value("code", self.code)
        writer.write_uuid_value("containerId", self.container_id)
        writer.write_str_value("description", self.description)
        writer.write_uuid_value("id", self.id)
        writer.write_collection_of_primitive_values("locationPaths", self.location_paths)
        writer.write_collection_of_primitive_values("locations", self.locations)
        writer.write_str_value("name", self.name)
        writer.write_float_value("originalAmount", self.original_amount)
        writer.write_datetime_value("plannedEndDate", self.planned_end_date)
        writer.write_datetime_value("plannedStartDate", self.planned_start_date)
        writer.write_float_value("quantity", self.quantity)
        writer.write_str_value("unit", self.unit)
        writer.write_float_value("unitPrice", self.unit_price)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .companies_get_response_results_addresses import CompaniesGetResponse_results_addresses
    from .companies_get_response_results_status import CompaniesGetResponse_results_status

@dataclass
class CompaniesGetResponse_results(Parsable):
    # The identifier of the account this company is associated with.
    account_id: Optional[UUID] = None
    # The company addresses.
    addresses: Optional[list[CompaniesGetResponse_results_addresses]] = None
    # The timestamp when this company was created.
    created_at: Optional[datetime.datetime] = None
    # The description of the company.Max length: 255
    description: Optional[str] = None
    # The ERP Partner Company ID.Max length: 255
    erp_id: Optional[str] = None
    # Id of the company.
    id: Optional[UUID] = None
    # The URL of the image associated to the company.Max length: 255
    image_url: Optional[str] = None
    # The name of the company. The company name should be unique under an account.Max length: 255
    name: Optional[str] = None
    # Original name of the company. Only returned when a company is deleted, since, in this case, the company ``name`` will be updated to ``removed at MMDDYYYY``.
    original_name: Optional[str] = None
    # The number of projects associated with the company.
    project_size: Optional[int] = None
    # The status of the company.Possible values: ``deleted``, ``active``
    status: Optional[CompaniesGetResponse_results_status] = None
    # The Tax ID.Max length: 255
    tax_id: Optional[str] = None
    # Trade or company type based on specialization.Max length: 255
    trade: Optional[str] = None
    # The timestamp when this company was last updated. This will only reflect changes to the company fields and not changes to any resources in the company.
    updated_at: Optional[datetime.datetime] = None
    # The number of users that are associated with the company.
    user_size: Optional[int] = None
    # The URL of the company website.Max length: 255
    website_url: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> CompaniesGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: CompaniesGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return CompaniesGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .companies_get_response_results_addresses import CompaniesGetResponse_results_addresses
        from .companies_get_response_results_status import CompaniesGetResponse_results_status

        from .companies_get_response_results_addresses import CompaniesGetResponse_results_addresses
        from .companies_get_response_results_status import CompaniesGetResponse_results_status

        fields: dict[str, Callable[[Any], None]] = {
            "accountId": lambda n : setattr(self, 'account_id', n.get_uuid_value()),
            "addresses": lambda n : setattr(self, 'addresses', n.get_collection_of_object_values(CompaniesGetResponse_results_addresses)),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "erpId": lambda n : setattr(self, 'erp_id', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "imageUrl": lambda n : setattr(self, 'image_url', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "originalName": lambda n : setattr(self, 'original_name', n.get_str_value()),
            "projectSize": lambda n : setattr(self, 'project_size', n.get_int_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(CompaniesGetResponse_results_status)),
            "taxId": lambda n : setattr(self, 'tax_id', n.get_str_value()),
            "trade": lambda n : setattr(self, 'trade', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "userSize": lambda n : setattr(self, 'user_size', n.get_int_value()),
            "websiteUrl": lambda n : setattr(self, 'website_url', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("accountId", self.account_id)
        writer.write_collection_of_object_values("addresses", self.addresses)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("description", self.description)
        writer.write_str_value("erpId", self.erp_id)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("imageUrl", self.image_url)
        writer.write_str_value("name", self.name)
        writer.write_str_value("originalName", self.original_name)
        writer.write_int_value("projectSize", self.project_size)
        writer.write_enum_value("status", self.status)
        writer.write_str_value("taxId", self.tax_id)
        writer.write_str_value("trade", self.trade)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_int_value("userSize", self.user_size)
        writer.write_str_value("websiteUrl", self.website_url)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .exports_post_response_status import ExportsPostResponse_status

@dataclass
class ExportsPostResponse(Parsable):
    # The ID of the export job. Use the ID to poll `GET exports/:exportId </en/docs/acc/v1/reference/http/sheets-exports-exportId-GET/>`_ to check the job's status. When the job is complete, you can retrieve the data you need to download the exported sheets.
    id: Optional[UUID] = None
    # The status of the export job; will always be: ``processing``
    status: Optional[ExportsPostResponse_status] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ExportsPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ExportsPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ExportsPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .exports_post_response_status import ExportsPostResponse_status

        from .exports_post_response_status import ExportsPostResponse_status

        fields: dict[str, Callable[[Any], None]] = {
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(ExportsPostResponse_status)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("id", self.id)
        writer.write_enum_value("status", self.status)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class WithWorkflowGetResponse_updateAttributesOptions(Parsable):
    """
    The configuration for applying attribute updates when a review is completed.This applies only if the workflow includes a file copy action and the ``Update Attributes`` action is enabled.
    """
    # (``Update attributes`` in the UI)Indicates whether the ``Update Attributes`` action is enabled.``true``: attributes will be applied after the review.``false``: (default) attributes will not be updated.
    enable_attached_attributes: Optional[bool] = None
    # (``Update attributes both for target folder and source folder`` or ``Update attributes only for target folder`` in the UI)Determines whether attributes are updated only for files in the target folder, or for both the target and source folders.``true``: update attributes in both folders.``false``: (default) update only the target folder.Only available when the approval workflow includes a copy post-action.
    update_source_and_copied_files: Optional[bool] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithWorkflowGetResponse_updateAttributesOptions:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithWorkflowGetResponse_updateAttributesOptions
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithWorkflowGetResponse_updateAttributesOptions()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "enableAttachedAttributes": lambda n : setattr(self, 'enable_attached_attributes', n.get_bool_value()),
            "updateSourceAndCopiedFiles": lambda n : setattr(self, 'update_source_and_copied_files', n.get_bool_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("enableAttachedAttributes", self.enable_attached_attributes)
        writer.write_bool_value("updateSourceAndCopiedFiles", self.update_source_and_copied_files)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID
from warnings import warn

if TYPE_CHECKING:
    from .get_filter_text_match_query_parameter_type import GetFilterTextMatchQueryParameterType
    from .projects_get_response import ProjectsGetResponse
    from .projects_post_request_body import ProjectsPostRequestBody
    from .projects_post_response import ProjectsPostResponse

class ProjectsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/admin/v1/accounts/{accountId}/projects
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new ProjectsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/admin/v1/accounts/{accountId}/projects{?fields*,filterTextMatch*,filter%5BbusinessUnitId%5D*,filter%5Bclassification%5D*,filter%5BjobNumber%5D*,filter%5Bname%5D*,filter%5Bplatform%5D*,filter%5Bproducts%5D*,filter%5Bstatus%5D*,filter%5Btype%5D*,filter%5BupdatedAt%5D*,limit*,offset*,sort*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[ProjectsRequestBuilderGetQueryParameters]] = None) -> Optional[ProjectsGetResponse]:
        """
        You can also use this endpoint to retrieve a list of project templates by setting the ``filter[classification]`` parameter to ``template`` or the ``filter[type]`` parameter to ``Template Project``.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[ProjectsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .projects_get_response import ProjectsGetResponse

        return await self.request_adapter.send_async(request_info, ProjectsGetResponse, None)
    
    async def post(self,body: ProjectsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[ProjectsPostResponse]:
        """
        Creates a new project in the specified account. You can create the project directly, or clone it from a project template.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[ProjectsPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .projects_post_response import ProjectsPostResponse

        return await self.request_adapter.send_async(request_info, ProjectsPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[ProjectsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        You can also use this endpoint to retrieve a list of project templates by setting the ``filter[classification]`` parameter to ``template`` or the ``filter[type]`` parameter to ``Template Project``.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: ProjectsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Creates a new project in the specified account. You can create the project directly, or clone it from a project template.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> ProjectsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: ProjectsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return ProjectsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class ProjectsRequestBuilderGetQueryParameters():
        """
        You can also use this endpoint to retrieve a list of project templates by setting the ``filter[classification]`` parameter to ``template`` or the ``filter[type]`` parameter to ``Template Project``.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filterbusiness_unit_id":
                return "filter%5BbusinessUnitId%5D"
            if original_name == "filterclassification":
                return "filter%5Bclassification%5D"
            if original_name == "filterjob_number":
                return "filter%5BjobNumber%5D"
            if original_name == "filtername":
                return "filter%5Bname%5D"
            if original_name == "filterplatform":
                return "filter%5Bplatform%5D"
            if original_name == "filterproducts":
                return "filter%5Bproducts%5D"
            if original_name == "filterstatus":
                return "filter%5Bstatus%5D"
            if original_name == "filtertype":
                return "filter%5Btype%5D"
            if original_name == "filterupdated_at":
                return "filter%5BupdatedAt%5D"
            if original_name == "filter_text_match":
                return "filterTextMatch"
            if original_name == "fields":
                return "fields"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # A comma-separated list of the project fields to include in the response. Default value: all fields.Possible values: ``accountId``, ``addressLine1``, ``addressLine2``, ``businessUnitId``, ``city``, ``companyCount``, ``constructionType``, ``country``, ``createdAt``, ``deliveryMethod``, ``endDate``, ``imageUrl``, ``jobNumber``, ``lastSignIn``, ``latitude``, ``longitude``, ``memberCount``, ``name``, ``platform``, ``postalCode``, ``products``, ``projectValue``, ``sheetCount``, ``startDate``, ``stateOrProvince``, ``status``, ``thumbnailImageUrl``, ``timezone``, ``type`` and ``updatedAt``.
        fields: Optional[list[str]] = None

        # Specifies how text-based filters should match values in supported fields.This parameter can be used in any endpoint that supports text-based filtering (e.g., ``filter[name]``, ``filter[jobNumber]``, ``filter[companyName]``, etc.).Possible values:``contains`` (default) – Matches if the field contains the specified text anywhere``startsWith`` – Matches if the field starts with the specified text``endsWith`` – Matches if the field ends with the specified text``equals`` – Matches only if the field exactly matches the specified textMatching is case-insensitive.Wildcards and regular expressions are not supported.
        filter_text_match: Optional[GetFilterTextMatchQueryParameterType] = None

        # The ID of the business unit that returned projects must be associated with.Note that you can obtain this ID value by calling the `GET business_units_structure </en/docs/acc/v1/reference/http/business_units_structure-GET/>`_ endpoint to retrieve a list of business units. Use the ``id`` field of the returned business unit that you want to filter by.Max length: 255
        filterbusiness_unit_id: Optional[UUID] = None

        # Filters projects by classification. Possible values:``production`` – Standard production projects.``template`` – Project templates that can be cloned to create production projects.``component`` – Placeholder projects that contain standardized components (e.g., forms) for use across projects. Only one component project is permitted per account. Known as a library in the ACC unified products UI.``sample`` – The single sample project automatically created upon ACC trial setup. Only one sample project is permitted per account.Max length: 255
        filterclassification: Optional[list[str]] = None

        # Filters by a user-defined project identifier. Supports partial matches when used with ``filterTextMatch``. For example, ``filter[jobNumber]=HP-0002&filterTextMatch=equals`` returns projects where the job number is exactly ``HP-0002``.Max length: 255
        filterjob_number: Optional[str] = None

        # Filters projects by name. Supports partial matches when used with ``filterTextMatch``. For example ``filter[name]=ABCco&filterTextMatch=startsWith`` returns projects whose names start with ``ABCco``.Max length: 255
        filtername: Optional[str] = None

        # Filters by platform. Possible values: ``acc`` (Autodesk Construction Cloud) and ``bim360`` (BIM 360).Max length: 255
        filterplatform: Optional[list[str]] = None

        # A comma-separated list of the products that the returned projects must use. Only projects that use one or more of the listed products are returned.Note that every product that can be used in a project on the same platform (ACC or BIM 360) is activated for the project. All products associated with the project are returned in the response.Some products are exclusive to ACC or to BIM 360, others are available on both platforms. Specify only the products on the appropriate platform for the projects you want to retrieve.Possible ACC values: ``accountAdministration``, ``autoSpecs``, ``build``, ``buildingConnected``, ``capitalPlanning``, ``cloudWorksharing``, ``cost``, ``designCollaboration``, ``docs``, ``financials``, ``insight``, ``modelCoordination``, ``projectAdministration``, ``takeoff``, and ``workshopxr``.Possible BIM 360 values: ``accountAdministration``, ``assets``, ``cloudWorksharing``, ``costManagement``, ``designCollaboration``, ``documentManagement``, ``field``, ``fieldManagement``, ``glue``, ``insight``, ``modelCoordination``, ``plan``, ``projectAdministration``, ``projectHome``, ``projectManagement``, and ``quantification``.
        filterproducts: Optional[list[str]] = None

        # Filters projects by status. Possible values: ``active``, ``pending``, ``archived``, ``suspended``.
        filterstatus: Optional[list[str]] = None

        # Filters by project type. To exclude a type, prefix it with ``-`` (e.g., ``-Bridge`` excludes bridge projects).Possible values: ``Airport``, ``Assisted Living / Nursing Home``, ``Bridge``, ``Canal / Waterway``, ``Convention Center``, ``Court House``, ``Data Center``, ``Dams / Flood Control / Reservoirs``, ``Demonstration Project``, ``Dormitory``, ``Education Facility``, ``Government Building``, ``Harbor / River Development``, ``Hospital``, ``Hotel / Motel``, ``Library``, ``Manufacturing / Factory``, ``Medical Laboratory``, ``Medical Office``, ``Military Facility``, ``Mining Facility``, ``Multi-Family Housing``, ``Museum``, ``Oil & Gas``,``Plant``, ``Office``, ``OutPatient Surgery Center``, ``Parking Structure / Garage``, ``Performing Arts``, ``Power Plant``, ``Prison / Correctional Facility``, ``Rail``, ``Recreation Building``, ``Religious Building``, ``Research Facility / Laboratory``, ``Restaurant``, ``Retail``, ``Seaport``, ``Single-Family Housing``, ``Solar Farm``, ``Stadium/Arena``, ``Streets / Roads / Highways``, ``Template Project``, ``Theme Park``, ``Training Project``, ``Transportation Building``, ``Tunnel``, ``Utilities``, ``Warehouse (non-manufacturing)``, ``Waste Water / Sewers``, ``Water Supply``, ``Wind Farm``.
        filtertype: Optional[list[str]] = None

        # Filters projects updated within a specific date range in ISO 8601 format. For example:Date range: ``2023-03-02T00:00:00.000Z..2023-03-03T23:59:59 .999Z``Specific start date: ``2023-03-02T00:00:00.000Z..``Specific end date: ``..2023-03-02T23:59:59.999Z``For more details, see `JSON API Filtering <https://jsonapi.org/format/#fetching-filtering>`_.Max length: 100
        filterupdated_at: Optional[str] = None

        # The maximum number of records to return in the response.Default: ``20``Minimum: ``1``Maximum: ``200`` (If a larger value is provided, only 200 records are returned)
        limit: Optional[int] = None

        # The index of the first record to return.Used for pagination in combination with the ``limit`` parameter.Example: ``limit=20`` and ``offset=40`` returns records 41–60.
        offset: Optional[int] = None

        # Sorts results by specified fields. Multiple fields can be used in order of priority. Each field can be followed by ``asc`` (ascending) or ``desc`` (descending). Default: ``asc``.For example, ``sort=name,createdAt desc`` sorts projects by name, then by creation date (newest first).Possible values: ``name`` (the default), ``startDate``, ``endDate``, ``type``, ``status``, ``jobNumber``, ``constructionType``, ``deliveryMethod``, ``contractType``, ``currentPhase``, ``companyCount``, ``memberCount``, ``createdAt`` and ``updatedAt``.
        sort: Optional[list[str]] = None

    
    @dataclass
    class ProjectsRequestBuilderGetRequestConfiguration(RequestConfiguration[ProjectsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class ProjectsRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .progress_get_response_results_candidates_companies import ProgressGetResponse_results_candidates_companies
    from .progress_get_response_results_candidates_roles import ProgressGetResponse_results_candidates_roles
    from .progress_get_response_results_candidates_users import ProgressGetResponse_results_candidates_users

@dataclass
class ProgressGetResponse_results_candidates(Parsable):
    """
    Information about the users, roles, and companies who are eligible to take the next action in this step.
    """
    # Companies that can act in this step.
    companies: Optional[list[ProgressGetResponse_results_candidates_companies]] = None
    # Project roles that can act in this step.
    roles: Optional[list[ProgressGetResponse_results_candidates_roles]] = None
    # Individual users who can act in this step.
    users: Optional[list[ProgressGetResponse_results_candidates_users]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ProgressGetResponse_results_candidates:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ProgressGetResponse_results_candidates
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ProgressGetResponse_results_candidates()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .progress_get_response_results_candidates_companies import ProgressGetResponse_results_candidates_companies
        from .progress_get_response_results_candidates_roles import ProgressGetResponse_results_candidates_roles
        from .progress_get_response_results_candidates_users import ProgressGetResponse_results_candidates_users

        from .progress_get_response_results_candidates_companies import ProgressGetResponse_results_candidates_companies
        from .progress_get_response_results_candidates_roles import ProgressGetResponse_results_candidates_roles
        from .progress_get_response_results_candidates_users import ProgressGetResponse_results_candidates_users

        fields: dict[str, Callable[[Any], None]] = {
            "companies": lambda n : setattr(self, 'companies', n.get_collection_of_object_values(ProgressGetResponse_results_candidates_companies)),
            "roles": lambda n : setattr(self, 'roles', n.get_collection_of_object_values(ProgressGetResponse_results_candidates_roles)),
            "users": lambda n : setattr(self, 'users', n.get_collection_of_object_values(ProgressGetResponse_results_candidates_users)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("companies", self.companies)
        writer.write_collection_of_object_values("roles", self.roles)
        writer.write_collection_of_object_values("users", self.users)
    




from enum import Enum

class GetFilterDataTypeQueryParameterType(str, Enum):
    List_ = "list",
    Dropdown = "dropdown",
    Text = "text",
    Paragraph = "paragraph",
    Numeric = "numeric",
    FilterDataTypeTextNumeric = "filter[dataType]=text,numeric",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class WithChangeOrderGetResponse_results_properties(Parsable):
    # A flag to indicate whether this is a pre-defined attribute or not. Inherited from the custom attribute definition.
    built_in: Optional[bool] = None
    # Name of the custom attribute. Inherited from the custom attribute definition.
    name: Optional[str] = None
    # The order of the custom attribute displayed in the BIM 360 Cost Management. Inherited from the custom attribute definition.
    position: Optional[float] = None
    # The ID of the custom attribute definition. This is the ID from ``/properties``.
    property_definition_id: Optional[UUID] = None
    # The type of the custom attribute. Inherited from the custom attribute definition.
    type: Optional[str] = None
    # The value of the custom attribute.
    value: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithChangeOrderGetResponse_results_properties:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithChangeOrderGetResponse_results_properties
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithChangeOrderGetResponse_results_properties()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "builtIn": lambda n : setattr(self, 'built_in', n.get_bool_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "position": lambda n : setattr(self, 'position', n.get_float_value()),
            "propertyDefinitionId": lambda n : setattr(self, 'property_definition_id', n.get_uuid_value()),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
            "value": lambda n : setattr(self, 'value', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("builtIn", self.built_in)
        writer.write_str_value("name", self.name)
        writer.write_float_value("position", self.position)
        writer.write_uuid_value("propertyDefinitionId", self.property_definition_id)
        writer.write_str_value("type", self.type)
        writer.write_str_value("value", self.value)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class TemplatesGetResponse(Parsable):
    # The date and time that the item was created, in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # The ID of the budget code template.
    id: Optional[UUID] = None
    # The state of the item during the integration with the external ERP system (such as SignNow). An item can be a ``budget``, ``contract``, ``main contract``, ``main contract item``, ``cost item``, ``expense``, ``expense item``, ``change order``, or ``schedule of value``. For more details, see `Integrate with External System </en/docs/acc/v1/tutorials/cost/integrate-with-external-system/>`_ tutorial.Possible values:``locked``: the item is currently locked within the ERP system, preventing modifications until unlocked. To unlock and modify the item, use the relevant PATCH endpoint to set ``integrationState`` to ``null``. For example, for a budget, call `PATCH budgets <en/docs/bim360/v1/reference/http/cost-budgets-budgetId-PATCH/>`_. For a contract, call `PATCH contracts <en/docs/bim360/v1/reference/http/cost-contracts-contractId-PATCH/>`_. For more details, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Integrated_and_Locked>`_.``integrated``: the item has been successfully added to the ERP system.``failed``: the item encountered an error during the integration process and was not successfully added to the ERP system. For example, if a user tries to integrate ``contracts`` from an ERP system and the updates fail, the ``integrationState`` can be set to ``failed``. Retry the sync process or analyze the issue if it continues to fail.``null``: The item has not been integrated with the ERP system. This is default value.For more information regarding integrations within the Cost Management system, see `Integrations in Cost Management <https://help.autodesk.com/view/BUILD/ENU/?guid=Cost_Integrations>`_.
    integration_state: Optional[str] = None
    # The date and time that the item's integration status was last changed.
    integration_state_changed_at: Optional[str] = None
    # The user who last changed the integration status. This is the ID of a user managed by the BIM 360/ACC Admin.
    integration_state_changed_by: Optional[str] = None
    # The lock status of template.
    locked: Optional[bool] = None
    # The name of the budget code template.Max length: 1024
    name: Optional[str] = None
    # The date and time that the item was last updated, in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> TemplatesGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: TemplatesGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return TemplatesGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "integrationState": lambda n : setattr(self, 'integration_state', n.get_str_value()),
            "integrationStateChangedAt": lambda n : setattr(self, 'integration_state_changed_at', n.get_str_value()),
            "integrationStateChangedBy": lambda n : setattr(self, 'integration_state_changed_by', n.get_str_value()),
            "locked": lambda n : setattr(self, 'locked', n.get_bool_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("integrationState", self.integration_state)
        writer.write_str_value("integrationStateChangedAt", self.integration_state_changed_at)
        writer.write_str_value("integrationStateChangedBy", self.integration_state_changed_by)
        writer.write_bool_value("locked", self.locked)
        writer.write_str_value("name", self.name)
        writer.write_datetime_value("updatedAt", self.updated_at)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .with_segment_get_response import WithSegmentGetResponse
    from .with_segment_patch_request_body import WithSegmentPatchRequestBody
    from .with_segment_patch_response import WithSegmentPatchResponse

class WithSegmentItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /cost/v1/containers/{containerId}/templates/{templateId}/segments/{segmentId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithSegmentItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/cost/v1/containers/{containerId}/templates/{templateId}/segments/{segmentId}{?force*}", path_parameters)
    
    async def delete(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> None:
        """
        Deletes a segment by ID.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: None
        """
        request_info = self.to_delete_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        return await self.request_adapter.send_no_response_content_async(request_info, None)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[WithSegmentGetResponse]:
        """
        Retrieves a segment by ID.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[WithSegmentGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .with_segment_get_response import WithSegmentGetResponse

        return await self.request_adapter.send_async(request_info, WithSegmentGetResponse, None)
    
    async def patch(self,body: WithSegmentPatchRequestBody, request_configuration: Optional[RequestConfiguration[WithSegmentItemRequestBuilderPatchQueryParameters]] = None) -> Optional[WithSegmentPatchResponse]:
        """
        Updates a segment by ID.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[WithSegmentPatchResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_patch_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .with_segment_patch_response import WithSegmentPatchResponse

        return await self.request_adapter.send_async(request_info, WithSegmentPatchResponse, None)
    
    def to_delete_request_information(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Deletes a segment by ID.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.DELETE, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        return request_info
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Retrieves a segment by ID.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_patch_request_information(self,body: WithSegmentPatchRequestBody, request_configuration: Optional[RequestConfiguration[WithSegmentItemRequestBuilderPatchQueryParameters]] = None) -> RequestInformation:
        """
        Updates a segment by ID.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.PATCH, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> WithSegmentItemRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: WithSegmentItemRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return WithSegmentItemRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class WithSegmentItemRequestBuilderDeleteRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class WithSegmentItemRequestBuilderGetRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class WithSegmentItemRequestBuilderPatchQueryParameters():
        """
        Updates a segment by ID.
        """
        # Force delete segment values when the segment length changes.``true``:  force delete,``false``:  not force delete.
        force: Optional[bool] = None

    
    @dataclass
    class WithSegmentItemRequestBuilderPatchRequestConfiguration(RequestConfiguration[WithSegmentItemRequestBuilderPatchQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class ImagePatchResponse(Parsable):
    # The account_id property
    account_id: Optional[UUID] = None
    # The address_line_1 property
    address_line_1: Optional[str] = None
    # The address_line_2 property
    address_line_2: Optional[str] = None
    # The business_unit_id property
    business_unit_id: Optional[UUID] = None
    # The city property
    city: Optional[str] = None
    # The construction_type property
    construction_type: Optional[str] = None
    # The contract_type property
    contract_type: Optional[str] = None
    # The country property
    country: Optional[str] = None
    # The currency property
    currency: Optional[str] = None
    # The end_date property
    end_date: Optional[str] = None
    # The id property
    id: Optional[UUID] = None
    # The job_number property
    job_number: Optional[str] = None
    # The language property
    language: Optional[str] = None
    # The last_sign_in property
    last_sign_in: Optional[datetime.datetime] = None
    # The name property
    name: Optional[str] = None
    # The postal_code property
    postal_code: Optional[str] = None
    # The project_type property
    project_type: Optional[str] = None
    # The start_date property
    start_date: Optional[str] = None
    # The state_or_province property
    state_or_province: Optional[str] = None
    # The status property
    status: Optional[str] = None
    # The timezone property
    timezone: Optional[str] = None
    # The value property
    value: Optional[float] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ImagePatchResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ImagePatchResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ImagePatchResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "account_id": lambda n : setattr(self, 'account_id', n.get_uuid_value()),
            "address_line_1": lambda n : setattr(self, 'address_line_1', n.get_str_value()),
            "address_line_2": lambda n : setattr(self, 'address_line_2', n.get_str_value()),
            "business_unit_id": lambda n : setattr(self, 'business_unit_id', n.get_uuid_value()),
            "city": lambda n : setattr(self, 'city', n.get_str_value()),
            "construction_type": lambda n : setattr(self, 'construction_type', n.get_str_value()),
            "contract_type": lambda n : setattr(self, 'contract_type', n.get_str_value()),
            "country": lambda n : setattr(self, 'country', n.get_str_value()),
            "currency": lambda n : setattr(self, 'currency', n.get_str_value()),
            "end_date": lambda n : setattr(self, 'end_date', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "job_number": lambda n : setattr(self, 'job_number', n.get_str_value()),
            "language": lambda n : setattr(self, 'language', n.get_str_value()),
            "last_sign_in": lambda n : setattr(self, 'last_sign_in', n.get_datetime_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "postal_code": lambda n : setattr(self, 'postal_code', n.get_str_value()),
            "project_type": lambda n : setattr(self, 'project_type', n.get_str_value()),
            "start_date": lambda n : setattr(self, 'start_date', n.get_str_value()),
            "state_or_province": lambda n : setattr(self, 'state_or_province', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_str_value()),
            "timezone": lambda n : setattr(self, 'timezone', n.get_str_value()),
            "value": lambda n : setattr(self, 'value', n.get_float_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("account_id", self.account_id)
        writer.write_str_value("address_line_1", self.address_line_1)
        writer.write_str_value("address_line_2", self.address_line_2)
        writer.write_uuid_value("business_unit_id", self.business_unit_id)
        writer.write_str_value("city", self.city)
        writer.write_str_value("construction_type", self.construction_type)
        writer.write_str_value("contract_type", self.contract_type)
        writer.write_str_value("country", self.country)
        writer.write_str_value("currency", self.currency)
        writer.write_str_value("end_date", self.end_date)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("job_number", self.job_number)
        writer.write_str_value("language", self.language)
        writer.write_datetime_value("last_sign_in", self.last_sign_in)
        writer.write_str_value("name", self.name)
        writer.write_str_value("postal_code", self.postal_code)
        writer.write_str_value("project_type", self.project_type)
        writer.write_str_value("start_date", self.start_date)
        writer.write_str_value("state_or_province", self.state_or_province)
        writer.write_str_value("status", self.status)
        writer.write_str_value("timezone", self.timezone)
        writer.write_float_value("value", self.value)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class CustomAttributesGetResponse_results_values(Parsable):
    # The time when the component was created (ISO8601 Date time format in UTC).
    created_at: Optional[str] = None
    # The actor that created the component. This is an Autodesk / Oxygen ID.
    created_by: Optional[str] = None
    # The ID of the custom attribute.
    custom_attribute_id: Optional[UUID] = None
    # The time when the component was deleted at (ISO8601 Date time format in UTC).
    deleted_at: Optional[str] = None
    # The actor that deleted the component. This is an Autodesk / Oxygen ID.
    deleted_by: Optional[str] = None
    # The display name of the value. Must be unique within the custom attribute (case-insensitive). Use togenerate the ``enumValues`` field for the custom attribute.
    display_name: Optional[str] = None
    # The ID of the component.
    id: Optional[str] = None
    # A flag indicating whether the component is active or inactive (``isActive`` is ``true`` if-and-only-if ``deletedAt`` is empty).
    is_active: Optional[bool] = None
    # The Autodesk Construction Cloud project ID. Must be a UUID or a project ID of the form ``b.{UUID}``.
    project_id: Optional[UUID] = None
    # The time when the component was last updated (ISO8601 Date time format in UTC).
    updated_at: Optional[str] = None
    # The actor that last updated the component. This is an Autodesk / Oxygen ID.
    updated_by: Optional[str] = None
    # A global sequence number that is incremented any time a component of this type is created, updated, or deleted. If you cache components, you can use the version value to compare the cached component to the same component online to see if the component has been updated. If the online component has a higher version value, it has been updated.
    version: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> CustomAttributesGetResponse_results_values:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: CustomAttributesGetResponse_results_values
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return CustomAttributesGetResponse_results_values()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_str_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "customAttributeId": lambda n : setattr(self, 'custom_attribute_id', n.get_uuid_value()),
            "deletedAt": lambda n : setattr(self, 'deleted_at', n.get_str_value()),
            "deletedBy": lambda n : setattr(self, 'deleted_by', n.get_str_value()),
            "displayName": lambda n : setattr(self, 'display_name', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "isActive": lambda n : setattr(self, 'is_active', n.get_bool_value()),
            "projectId": lambda n : setattr(self, 'project_id', n.get_uuid_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_str_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "version": lambda n : setattr(self, 'version', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_uuid_value("customAttributeId", self.custom_attribute_id)
        writer.write_str_value("deletedAt", self.deleted_at)
        writer.write_str_value("deletedBy", self.deleted_by)
        writer.write_str_value("displayName", self.display_name)
        writer.write_str_value("id", self.id)
        writer.write_bool_value("isActive", self.is_active)
        writer.write_uuid_value("projectId", self.project_id)
        writer.write_str_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_int_value("version", self.version)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .properties_get_response_bbox_max import PropertiesGetResponse_bboxMax
    from .properties_get_response_bbox_min import PropertiesGetResponse_bboxMin
    from .properties_get_response_props import PropertiesGetResponse_props
    from .properties_get_response_props_ignored import PropertiesGetResponse_propsIgnored

@dataclass
class PropertiesGetResponse(Parsable):
    # maximum [x, y, z]-coords of the 3D-bbox.
    bbox_max: Optional[PropertiesGetResponse_bboxMax] = None
    # minimum [x, y, z]-coords of the 3D-bbox.
    bbox_min: Optional[PropertiesGetResponse_bboxMin] = None
    # Property database id (the property database the object originated from).
    db_id: Optional[str] = None
    # The external ID of the object.
    external_id: Optional[str] = None
    # Hash used to determine whether the object geometry has changed between different versions.
    geom_hash: Optional[str] = None
    # The lineage ID of the object; the (svf2Id, lineageId)-pair allows to track a specific object across several versions
    lineage_id: Optional[str] = None
    # Object database id from the original seed file property database (_objects_id.id).
    lmv_id: Optional[int] = None
    # Property database property keyed values.
    props: Optional[PropertiesGetResponse_props] = None
    # Hash used to determine whether object properties have changed between different versions.
    props_hash: Optional[str] = None
    # Property database property values that are not considered for property change tracking.
    props_ignored: Optional[PropertiesGetResponse_propsIgnored] = None
    # The stable SVF2 ID of the object.
    svf2_id: Optional[int] = None
    # List of corresponding view IDs in the index manifest that this object is visible in.
    views: Optional[list[str]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PropertiesGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PropertiesGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PropertiesGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .properties_get_response_bbox_max import PropertiesGetResponse_bboxMax
        from .properties_get_response_bbox_min import PropertiesGetResponse_bboxMin
        from .properties_get_response_props import PropertiesGetResponse_props
        from .properties_get_response_props_ignored import PropertiesGetResponse_propsIgnored

        from .properties_get_response_bbox_max import PropertiesGetResponse_bboxMax
        from .properties_get_response_bbox_min import PropertiesGetResponse_bboxMin
        from .properties_get_response_props import PropertiesGetResponse_props
        from .properties_get_response_props_ignored import PropertiesGetResponse_propsIgnored

        fields: dict[str, Callable[[Any], None]] = {
            "bboxMax": lambda n : setattr(self, 'bbox_max', n.get_object_value(PropertiesGetResponse_bboxMax)),
            "bboxMin": lambda n : setattr(self, 'bbox_min', n.get_object_value(PropertiesGetResponse_bboxMin)),
            "dbId": lambda n : setattr(self, 'db_id', n.get_str_value()),
            "externalId": lambda n : setattr(self, 'external_id', n.get_str_value()),
            "geomHash": lambda n : setattr(self, 'geom_hash', n.get_str_value()),
            "lineageId": lambda n : setattr(self, 'lineage_id', n.get_str_value()),
            "lmvId": lambda n : setattr(self, 'lmv_id', n.get_int_value()),
            "props": lambda n : setattr(self, 'props', n.get_object_value(PropertiesGetResponse_props)),
            "propsHash": lambda n : setattr(self, 'props_hash', n.get_str_value()),
            "propsIgnored": lambda n : setattr(self, 'props_ignored', n.get_object_value(PropertiesGetResponse_propsIgnored)),
            "svf2Id": lambda n : setattr(self, 'svf2_id', n.get_int_value()),
            "views": lambda n : setattr(self, 'views', n.get_collection_of_primitive_values(str)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("bboxMax", self.bbox_max)
        writer.write_object_value("bboxMin", self.bbox_min)
        writer.write_str_value("dbId", self.db_id)
        writer.write_str_value("externalId", self.external_id)
        writer.write_str_value("geomHash", self.geom_hash)
        writer.write_str_value("lineageId", self.lineage_id)
        writer.write_int_value("lmvId", self.lmv_id)
        writer.write_object_value("props", self.props)
        writer.write_str_value("propsHash", self.props_hash)
        writer.write_object_value("propsIgnored", self.props_ignored)
        writer.write_int_value("svf2Id", self.svf2_id)
        writer.write_collection_of_primitive_values("views", self.views)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .versions_get_response_results_review_content_custom_attributes import VersionsGetResponse_results_reviewContent_customAttributes

@dataclass
class VersionsGetResponse_results_reviewContent(Parsable):
    """
    Review-specific metadata related to the file version.
    """
    # A list of custom attributes applied to the file during the review.
    custom_attributes: Optional[list[VersionsGetResponse_results_reviewContent_customAttributes]] = None
    # The pending file name assigned during the review.If the review is approved, this becomes the official version name.In most cases, this value is automatically generated according to the naming standard configured on the folder.
    name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> VersionsGetResponse_results_reviewContent:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: VersionsGetResponse_results_reviewContent
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return VersionsGetResponse_results_reviewContent()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .versions_get_response_results_review_content_custom_attributes import VersionsGetResponse_results_reviewContent_customAttributes

        from .versions_get_response_results_review_content_custom_attributes import VersionsGetResponse_results_reviewContent_customAttributes

        fields: dict[str, Callable[[Any], None]] = {
            "customAttributes": lambda n : setattr(self, 'custom_attributes', n.get_collection_of_object_values(VersionsGetResponse_results_reviewContent_customAttributes)),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("customAttributes", self.custom_attributes)
        writer.write_str_value("name", self.name)
    

